<?php
/*
Plugin Name: WyPiekacz
Plugin URI: http://www.poradnik-webmastera.com/projekty/wypiekacz
Description: Checks if posts submitted for review and posted satisfies set of rules
Author: Daniel Frużyński
Version: 1.2
Author URI: http://www.poradnik-webmastera.com/
Text Domain: wypiekacz
*/

/*  Copyright 2009  Daniel Frużyński  (email : daniel [A-T] poradnik-webmastera.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


if ( !class_exists( 'WyPiekacz' ) ) {
	class WyPiekacz {
		var $errors = array();
		
		// Constructor
		function WyPiekacz() {
			add_filter( 'wp_insert_post_data', array( &$this, 'wp_insert_post' ), 100, 2 );
			add_action( 'save_post', array( &$this, 'save_post' ) );
			add_action( 'admin_notices', array( &$this, 'admin_notice' ) );
			
			add_action( 'submitpost_box', array( &$this, 'submitpost_box' ) );
			
			add_action( 'admin_menu', array( &$this, 'admin_menu' ) );
			
			add_action( 'init', array( &$this, 'init' ) );
		}
		
		function init() {
			if ( function_exists( 'load_plugin_textdomain' ) ) {
				load_plugin_textdomain( 'wypiekacz', PLUGINDIR.'/'.dirname(plugin_basename(__FILE__)) );
			}
		}
		
		function admin_notice() {
			global $post;
			$meta = '';
			if ( is_object( $post) ) {
				$meta = get_post_meta($post->ID, 'WyPiekacz_msg', true);
			}
			if ('' != $meta ) {
				// Display error message
				echo '<div id="notice" class="error"><p>', $meta, 
					'<br />', __('Post Status has been changed to Draft.', 'wypiekacz'), '</p></div>', "\n";
				// Remove this message
				delete_post_meta( $post->ID, 'WyPiekacz_msg' );
				// Change WP message to 'Post saved'
				if ( isset( $_GET['message'] ) ) {
					if ( '6' == $_GET['message'] ) {
						$_GET['message'] = '7';
					}
				}
			}
		}
		
		function wp_insert_post( $data, $post_arr ) {
			if (
				// Check only for posts
				( 'post' == $data['post_type'])
				// Check only if status is Published or Pending Review
				&& ( ( 'publish' == $data['post_status'] ) || ( 'pending' == $data['post_status'] ) )
				// Editors (and above) can have limits too
				&& ( ( !get_option( 'wypiekacz_limit_for_all' ) && !current_user_can( 'edit_others_posts' ) )
					|| get_option( 'wypiekacz_limit_for_all' ) )
			) {
				$result = $this->check_precel_post( $data['post_content'], $data['post_title'], $post_arr );
				if ( true !== $result ) {
					// Revert post status to Draft
					$data['post_status'] = 'draft';
				}
			}
			return $data;
		}
		
		function save_post( $post_ID ) {
			if ( count( $this->errors ) > 0 ) {
				delete_post_meta( $post_ID, 'WyPiekacz_msg' );
				add_post_meta( $post_ID, 'WyPiekacz_msg', implode( '<br />', $this->errors ), true );
			}
		}
		
		function check_precel_post( $text, $title, $post_data ) {
			// Check length
			$min_len = get_option( 'wypiekacz_min_len' );
			$text2 = preg_replace( '/\s\s+/', ' ', trim( strip_tags( $text ) ) );
			$len = strlen( $text2 );
			if ( $len < $min_len ) {
				$this->errors[] = sprintf( __('Post is too short (minimum is %1$s chars, your post has %2$s).', 'wypiekacz'),
					$min_len, $len );
			}
			
			// Check links
			$max_links = get_option( 'wypiekacz_max_links' );
			$cnt = preg_match_all( '/<\s*a\s/i', $text, $matches );
			if ( $cnt > $max_links ) {
				$this->errors[] = sprintf( __('Post contains too many links (maximum is %1$s, your post has %2$s).', 'wypiekacz'),
					$max_links, $cnt );
			}
			
			// Check title length
			$min_len = get_option( 'wypiekacz_min_title_len' );
			$text2 = trim( $title );
			$len = strlen( $text2 );
			if ( $len < $min_len ) {
				$this->errors[] = sprintf( __('Post Title is too short (minimum is %1$s chars, your Title has %2$s).', 'wypiekacz'),
					$min_len, $len );
			}
			
			// Get categories from POST data
			if ( empty( $post_data['post_category'] ) || ( 0 == count( $post_data['post_category'] ) ) 
				|| !is_array( $post_data['post_category'] ) ) {
				$post_cat_cnt = 0;
				$categories = array();
			} else {
				$post_cat_cnt = count( $post_data['post_category'] );
				$categories = $post_data['post_category'];
			}
			
			$has_default_cat = false;
			$default_cat = get_option( 'default_category' );
			foreach ( $categories as $category ) {
				if ( $category == $default_cat ) {
					$has_default_cat = true;
					break;
				}
			}
			
			// Check default category
			$use_default_cat = get_option( 'wypiekacz_use_def_cat' );
			if ( !$use_default_cat && $has_default_cat ) {
				$this->errors[] = sprintf( __('Cannot add posts to the default category (%s).', 'wypiekacz'), 
					get_cat_name( $default_cat ) );
			}
			
			// When post doesn't have categories, default one will be used
			if ( ( 0 == $post_cat_cnt ) && $use_default_cat ) {
				$post_cat_cnt = 1;
			}
			
			// Check categories count
			$min_cats = get_option( 'wypiekacz_min_cats' );
			$max_cats = get_option( 'wypiekacz_max_cats' );
			if ( $post_cat_cnt < $min_cats ) {
				$this->errors[] = sprintf( __('Too few categories selected (minimum is %1$s, your post has %2$s).', 'wypiekacz'),
					$min_cats, $post_cat_cnt );
			} else if ( $post_cat_cnt > $max_cats ) {
				$this->errors[] = sprintf( __('Too many categories selected (maximum is %1$s, your post has %2$s).', 'wypiekacz'),
					$max_cats, $post_cat_cnt );
			}
			
			// Get categories from POST data
			if ( empty( $post_data['tags_input'] ) ) {
				$post_tag_cnt = 0;
			} else {
				$post_tag_cnt = count( explode( ',', $post_data['tags_input'] ) );
			}
			
			// Check tag count
			$min_tags = get_option( 'wypiekacz_min_tags' );
			$max_tags = get_option( 'wypiekacz_max_tags' );
			
			if ( $post_tag_cnt < $min_tags ) {
				$this->errors[] = sprintf( __('Too few tags (minimum is %1$s, your post has %2$s).', 'wypiekacz'),
					$min_tags, $post_tag_cnt );
			} else if ($post_tag_cnt > $max_tags) {
				$this->errors[] = sprintf( __('Too many tags (maximum is %1$s, your post has %2$s).', 'wypiekacz'),
					$max_tags, $post_tag_cnt );
			}
			
			return count( $this->errors ) == 0;
		}
		
		function submitpost_box() {
			global $post;
			
			// Title
			if ( empty( $post->post_title ) || ( '' == $post->post_title ) ) {
				$post->post_title = get_option( 'wypiekacz_def_title' );
			}
			
			// Text
			if ( empty( $post->post_content ) || ( '' == $post->post_content ) ) {
				$post->post_content = get_option( 'wypiekacz_def_text' );
			}
		}
		
		function admin_menu() {
			$file = __FILE__;
			
			// hack for 1.5
			global $wp_version;
			if ( '1.5' == substr( $wp_version, 0, 3 ) ) {
				$file = 'wypiekacz/wypiekacz.php';
			}
			add_submenu_page( 'options-general.php', 'WyPiekacz', 
				'WyPiekacz', 10, $file, array( $this, 'options_panel' ) );
		}
		
		function options_panel() {
			$message = null;
			if ( isset($_POST['action']) ) {
				$message = __('Configuration has been saved.', 'wypiekacz');
				update_option( 'wypiekacz_min_len', intval( $_POST['wypiekacz_min_len'] ) );
				update_option( 'wypiekacz_max_links', intval( $_POST['wypiekacz_max_links'] ) );
				update_option( 'wypiekacz_min_title_len', intval( $_POST['wypiekacz_min_title_len'] ) );
				update_option( 'wypiekacz_use_def_cat', isset( $_POST['wypiekacz_use_def_cat'] ) ? 1 : 0 );
				update_option( 'wypiekacz_min_cats', intval( $_POST['wypiekacz_min_cats'] ) );
				update_option( 'wypiekacz_max_cats', intval( $_POST['wypiekacz_max_cats'] ) );
				update_option( 'wypiekacz_min_tags', intval( $_POST['wypiekacz_min_tags'] ) );
				update_option( 'wypiekacz_max_tags', intval( $_POST['wypiekacz_max_tags'] ) );
				update_option( 'wypiekacz_limit_for_all', isset( $_POST['wypiekacz_limit_for_all'] ) ? 1 : 0 );
				update_option( 'wypiekacz_def_title', $_POST['wypiekacz_def_title'] );
				update_option( 'wypiekacz_def_text', $_POST['wypiekacz_def_text'] );
			}
			
			if ($message) {
				echo '<div id="message" class="updated fade"><p>', $message, '</p></div>', "\n";
			}
?>
<div id="dropmessage" class="updated" style="display:none;"></div>
<div class="wrap">
<h2><?php _e('WyPiekacz - Options', 'wypiekacz'); ?></h2>

<form name="dofollow" action="" method="post">
<table class="form-table">

<tr><th colspan="2"><h3><?php _e('Post:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_len"><?php _e('Minimum post length (in chars):', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="6" size="10" id="wypiekacz_min_len" name="wypiekacz_min_len" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_len' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_max_links"><?php _e('Maximum link count in post:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="2" size="10" id="wypiekacz_max_links" name="wypiekacz_max_links" value="<?php echo stripcslashes( get_option( 'wypiekacz_max_links' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_title_len"><?php _e('Minimum title length (in chars):', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_min_title_len" name="wypiekacz_min_title_len" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_title_len' ) ); ?>" />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Categories:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_use_def_cat"><?php printf( __('Posts can be added to the default category (%s):', 'wypiekacz'), get_cat_name( get_option( 'default_category' ) ) ); ?></label>
</th>
<td>
<input type="checkbox" maxlength="3" id="wypiekacz_use_def_cat" name="wypiekacz_use_def_cat" value="yes" <?php echo get_option( 'wypiekacz_use_def_cat' ) ? 'checked="checked"' : ''; ?> />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_cats"><?php _e('Minimum categories selected:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_min_cats" name="wypiekacz_min_cats" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_cats' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_max_cats"><?php _e('Maximum categories selected:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_max_cats" name="wypiekacz_max_cats" value="<?php echo stripcslashes( get_option( 'wypiekacz_max_cats' ) ); ?>" />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Tags:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_min_tags"><?php _e('Minimum tag count:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_min_tags" name="wypiekacz_min_tags" value="<?php echo stripcslashes( get_option( 'wypiekacz_min_tags' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_max_tags"><?php _e('Maximum tag count:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" maxlength="3" size="10" id="wypiekacz_max_tags" name="wypiekacz_max_tags" value="<?php echo stripcslashes( get_option( 'wypiekacz_max_tags' ) ); ?>" />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Special:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_limit_for_all"><?php _e('Check above rules for Editors and Administrators:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="checkbox" maxlength="3" id="wypiekacz_limit_for_all" name="wypiekacz_limit_for_all" value="yes" <?php echo get_option( 'wypiekacz_limit_for_all' ) ? 'checked="checked"' : ''; ?> />
</td>
</tr>

<tr><th colspan="2"><h3><?php _e('Default post template:', 'wypiekacz'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_def_title"><?php _e('Title:', 'wypiekacz'); ?></label>
</th>
<td>
<input type="text" size="59" id="wypiekacz_def_title" name="wypiekacz_def_title" value="<?php echo stripcslashes( get_option( 'wypiekacz_def_title' ) ); ?>" />
</td>
</tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="wypiekacz_def_text"><?php _e('Text:', 'wypiekacz'); ?></label>
</th>
<td>
<textarea rows="5" cols="57" id="wypiekacz_def_text" name="wypiekacz_def_text"><?php echo stripcslashes( get_option( 'wypiekacz_def_text' ) ); ?></textarea>
</td>
</tr>

</table>

<p class="submit">
<input type="hidden" name="action" value="update" />
<input type="submit" name="Submit" value="<?php _e('Save settings', 'wypiekacz'); ?>" /> 
</p>

</form>
</div>
<?php
		}
	}
	
	add_option( 'wypiekacz_min_len', 1000, 'Minimum post length', 'yes' );
	add_option( 'wypiekacz_max_links', 3, 'Maximum links per post', 'yes' );
	add_option( 'wypiekacz_min_title_len', 5, 'Minimum title length', 'yes' );
	add_option( 'wypiekacz_use_def_cat', 1, 'Posts can be added to the default category', 'yes' );
	add_option( 'wypiekacz_min_cats', 1, 'Minimum category count', 'yes' );
	add_option( 'wypiekacz_max_cats', 3, 'Maximum category count', 'yes' );
	add_option( 'wypiekacz_min_tags', 1, 'Minimum tag count', 'yes' );
	add_option( 'wypiekacz_max_tags', 5, 'Maximum tag count', 'yes' );
	add_option( 'wypiekacz_limit_for_all', 1, 'Check ruled for Editors and Administrators', 'yes' );
	add_option( 'wypiekacz_def_title', '', 'Default post title', 'yes' );
	add_option( 'wypiekacz_def_text', '', 'Default post text', 'yes' );
	
	$wp_wypiekacz = new WyPiekacz();
}

?>
